/*This program is provided by Tabitha Garwe, PhD */;

/*Part I: Propensity-adjusted analysis - Regression (Covariance Adjustment)*/;

/* Part Ia: Assess interaction between PS and treatmentment variable (LSI) --> 
Breslow Day Test - Homogeneity of OR across quintiles*/;
ods rtf file='Breslow.rtf';
proc freq data=ranks;
tables quintile*died;
run;


proc freq data=ranks;
  table quintile*LSI*Died/ relrisk cmh norow nocol nopercent ;
run;
ods rtf close;

/*Part Ib: If no interaction between PS and treatmentment variable, we can proceed with regression adjustment;
The PS variable can be included as a composite confounder in multivariable analysis (Logistic regression, Cox regression, Poisson, etc.);
The regression analysis may include only the treatment variable and the propensity score as covariates, or additional important covariates.*/;
ods rtf file='PSAdjusted.rtf';
proc logistic descending data=ranks;
class  GE55 (descending param=ref) Male(descending param=ref ) VarPM(descending param=ref) VarLV(descending param=ref) 
VarINT(descending param=ref) VarHS(descending param=ref ) VarCM (descending param=ref) VarCD (descending param=ref) VarSVH (descending param=ref)
VarTS(descending param=ref)  LSI(descending param=ref);
*Model died = LSI quintile LSI*Quintile/lackfit risklimits; /***INTERACTION MODEL*/;
*Model died = LSI quintile DSev1 DSev2  LSI*Quintile LSI*Dsev1 LSI*DSev2 Dsev1*Dsev2/lackfit risklimits; /*** INTERACTION MODEL */;
*Model died = LSI pscore DSev1 DSev2   /lackfit risklimits; /***REDUCED FINAL MODEL*/;
Model died = LSI quintile DSev1 DSev2 DSev1F  /lackfit risklimits; /***REDUCED FINAL MODEL*/;
*Model died = LSI pscore DSev1 DSev2 DSev1F  /lackfit risklimits; /***REDUCED FINAL MODEL*/;
roc; roccontrast;
run;
ods rtf close;

/*PART II: Propensity-adjusted analysis - Stratification

/Part 2a - Calculate pooled estimate if no interaction present*/;

ods rtf file='Pooled.rtf';
proc freq data=ranks;
  table quintile*LSI*Died/ relrisk cmh norow nocol nopercent ;
  *where quintile = 4;
run;

ods rtf close;

/*2b - Stratified multivariable analyisis (Regression-Stratification)*/;
/*ATE*/;

ods rtf file='PSAStratify.rtf';
proc logistic descending data=ranks;
class  GE55 (descending param=ref) Male(descending param=ref ) VarPM(descending param=ref) VarLV(descending param=ref) 
VarINT(descending param=ref) VarHS(descending param=ref ) VarCM (descending param=ref) VarCD (descending param=ref) VarSVH (descending param=ref)
VarTS(descending param=ref)  LSI(descending param=ref);

*Model died = LSI  DSev1 DSev2  / risklimits; /***REDUCED FINAL MODEL*/;
Model died = LSI DSev1 DSev2 DSev1F  /risklimits; /***REDUCED FINAL MODEL*/;
strata quintile;
run;
ods rtf close;




Data two;
set ranks;
if LSI=1 then Interv='Yes'; else Interv='No';
if 18 le ageP le 29 then agegrp=1;
  else if 30 le ageP le 44 then agegrp=2;
else if 45 le ageP le 54 then agegrp=3;
else if 55 le ageP le 64 then agegrp=4;
else if ageP ge 65 then agegrp=5;
run;




/*PART III: PROPENSITY SCORE MATCHING*/;

/*The following programs are adapted from PROC PSMATCH  - https://support.sas.com/documentation/onlinedoc/stat/142/psmatch.pdf*/;

/*The CALIPER=0.5 option requests that a match be made only if the difference in the logits of the propensity
scores for pairs of individuals is less than or equal to 0.5 times the pooled estimate of the common standard
deviation of the logits of the propensity scores*/;


//*PART IIIa: Mahalanobis Distance Matching within calipers  - Greedy matching */;

ods graphics on;
proc psmatch data=TWO region=treated;
class  GE55  Male VarPM VarLV VarINT VarHS VarCM  VarCD  VarSVH 
VarTS  Interv agegrp;
psmodel Interv (Treated='Yes')= GE55  Male VarPM VarLV VarINT VarHS VarCM  VarCD  VarSVH 
VarTS AgeP DSev1 DSev2 dsev1*dsev2;
match method=greedy(k=1)  exact=Male exact=agegrp
	stat=mah(lps var=(Dsev2 DSev1F ageP)) caliper= 0.5;
assess  lps var= (VarPM VarINT VarHS VarCM  VarCD  VarSVH 
VarTS AgeP DSev1 DSev2 DSev1F) / plots=(barchart boxplot ) weight=none;
output out(obs=match)=matched_Mh lps=_Lps matchid=_MatchID;
run;
proc sort  data=matched_Mh;
by _matchid;
run;
ods rtf file = 'MHMatched.rtf';
proc print data=matched_Mh;
var LSI DSev1 DSev2 DSev1F AgeP _ps_ _lps _matchid;
where _matchid <=5;
run;

Proc freq;
tables LSI*died;
RUN;
ods rtf close;

/*Unconditial logistic regression*/;
proc logistic descending data=matched_Mh;
class   LSI(descending param=ref);
*Model died = LSI  / risklimits; 
Model died = LSI  DSev2 DSev1F  / risklimits; 
run;
/*Conditial logistic regression */;
proc logistic descending data=matched_Mh;
class  LSI(descending param=ref);
*Model died = LSI  / risklimits; 
Model died = LSI DSev2 DSev1F  / risklimits; 
strata _matchid;
run;




/*Part IIIb: Optimal Fixed Ratio Matching(1:1) with logit of propensity score - no defined caliper*/;

ods graphics on;
proc psmatch data=TWO region=allobs;
class  GE55  Male VarPM VarLV VarINT VarHS VarCM  VarCD  VarSVH 
VarTS  Interv agegrp;
psmodel Interv (Treated='Yes')= GE55  Male VarPM VarLV VarINT VarHS VarCM  VarCD  VarSVH 
VarTS AgeP DSev1 DSev2 dsev1*dsev2;
match method=optimal(k=1)  exact=Male exact=agegrp distance=lps caliper=.;
assess  ps var= (VarPM VarINT VarHS VarCM  VarCD  VarSVH 
VarTS AgeP DSev1 DSev2) / plots=(barchart boxplot) weight=none;
output out(obs=match)=matchOFR lps=_Lps matchid=_MatchID;
run;
proc sort  data=MatchOFR;
by _matchid;
run;
ods rtf file = 'FRMatched.rtf';
proc print data=MatchOFR;
var LSI DSev1 DSev2 DSev1F AgeP _ps_ _lps _matchid;
where _matchid <=5;
run;
ods rtf close;

Proc freq data=MatchOFR;
tables LSI*died;
RUN;



/*PART IIIc: Greedy Matching within calipers defined by logit of propensity score*/;

ods graphics on;
proc psmatch data=TWO region=allobs;
class  GE55  Male VarPM VarLV VarINT VarHS VarCM  VarCD  VarSVH 
VarTS  Interv agegrp;
psmodel Interv (Treated='Yes')= GE55  Male VarPM VarLV VarINT VarHS VarCM  VarCD  VarSVH 
VarTS AgeP DSev1 DSev2 dsev1*dsev2;
match method=greedy(k=1)  exact=Male exact=agegrp distance=lps caliper= 0.25;
assess  ps lps var= (VarPM VarINT VarHS VarCM  VarCD  VarSVH 
VarTS AgeP DSev1 DSev2 Dsev1F) / plots=(barchart ) weight=none;
output out(obs=match)=matchGRD lps=_Lps matchid=_MatchID;
run;
proc sort  data=MatchGRD;
by _matchid;
run;
ods rtf file = 'GRDMatched.rtf';
proc print data=MatchGRD;
var LSI DSev1 DSev2 DSev1F AgeP _ps_ _lps _matchid;
where _matchid <=5;
run;

Proc freq data=MatchGRD;
tables LSI*died;
RUN;
ods rtf close;


/*Unconditial logistic regression*/;
proc logistic descending data=matchGRD;
class   LSI(descending param=ref);
*Model died = LSI  / risklimits; 
Model died = LSI dsev2  DSev1F  / risklimits; 
run;
/*Conditial logistic regression */;
proc logistic descending data=matchGRD;
class  LSI(descending param=ref);
*Model died = LSI  / risklimits; 
Model died = LSI  dsev2 DSev1F  / risklimits; 
strata _matchid;
run;




/*TIME-PERMITTING*/;

/*PART IV: Propensity Score Weighting*/;


/****ATT Weighting***/;

ods graphics on;
proc psmatch data=TWO region=allobs;
class  GE55  Male VarPM VarLV VarINT VarHS VarCM  VarCD  VarSVH 
VarTS  Interv agegrp;
psmodel Interv (Treated='Yes')= GE55  Male VarPM VarLV VarINT VarHS VarCM  VarCD  VarSVH 
VarTS AgeP DSev1 DSev2 dsev1*dsev2;
assess  ps lps var= (VarPM VarINT VarHS VarCM  VarCD  VarSVH 
VarTS AgeP DSev1 DSev2 Dsev1F) / plots=(barchart boxplot) weight=attwgt NLARGESTWGT=6;
output out(obs=ALL)=ATTWT;
run;

ods rtf file = 'ATTWGT.rtf';
proc surveylogistic data = attwt;
class lsi (descending param=ref);
weight _ATTWGT_;
Model died (desc) = LSI DSev1F  / expb clodds rsquare; 
*Model died (desc) = LSI  dsev2 DSev1F  / expb clodds rsquare; 
run;
ods rtf close;


/****ATE Weighting***/;

ods graphics on;
proc psmatch data=TWO region=allobs;
class  GE55  Male VarPM VarLV VarINT VarHS VarCM  VarCD  VarSVH 
VarTS  Interv agegrp;
psmodel Interv (Treated='Yes')= GE55  Male VarPM VarLV VarINT VarHS VarCM  VarCD  VarSVH 
VarTS AgeP DSev1 DSev2 dsev1*dsev2;
assess  ps lps var= (VarPM VarINT VarHS VarCM  VarCD  VarSVH 
VarTS AgeP DSev1 DSev2 Dsev1F) / plots=(barchart boxplot) weight=ATEWGT NLARGESTWGT=6 ;
output out(obs=ALL)=ATEWT atewgt=ate;
run;
ods rtf file = 'ATEWGT.rtf';
proc surveylogistic data = atewt;
class lsi (descending param=ref);
weight ATE;
*Model died (desc) = LSI   / expb clodds rsquare; 
Model died (desc) = LSI   DSev1F  / expb clodds rsquare; 
run;
ods rtf close;


/****STABILIZED ATE Weighting ***/;
ods graphics on;
proc psmatch data=TWO region=allobs;
class  GE55  Male VarPM VarLV VarINT VarHS VarCM  VarCD  VarSVH 
VarTS  Interv agegrp;
psmodel Interv (Treated='Yes')= GE55  Male VarPM VarLV VarINT VarHS VarCM  VarCD  VarSVH 
VarTS AgeP DSev1 DSev2 dsev1*dsev2;
assess  ps lps var= (VarPM VarINT VarHS VarCM  VarCD  VarSVH 
VarTS AgeP DSev1 DSev2 Dsev1F) / plots=(barchart boxplot) weight=ATEWGT (STABILIZE=YES) NLARGESTWGT=3;
output out(obs=ALL)=ATEWT2 atewgt=ate_S;
RUN;

ods rtf file = 'STATEWGT.rtf';
proc surveylogistic data = atewt2;
class lsi (descending param=ref);
weight ATE_S;
*Model died (desc) = LSI   / expb clodds rsquare; 
Model died (desc) = LSI DSev1F  / expb clodds rsquare; 
run;
ods rtf close;


/*SUPPLEMENTAL SAMPLE PROGRAMS*/;

/*PART II Sample program(s)*/;

/*PART 2: Propensity Score Stratification  and Weighting using PROC PSMATCH*/;
/****SAMPLE CODE ONLY***/;
ods graphics on;
proc psmatch data=TWO region=allobs;
class  GE55  Male VarPM VarLV VarINT VarHS VarCM  VarCD  VarSVH 
VarTS  Interv;
psmodel Interv (Treated='Yes')= GE55  Male VarPM VarLV VarINT VarHS VarCM  VarCD  VarSVH 
VarTS AgeP DSev1 DSev2;
strata nstrata=5  ;
assess ps var=(DSev1 Dsev2)
/ varinfo stratumwgt=TOTAL plots=(barchart );
output out(obs=all)=strat ATEWGT=stratumwgt;
run;
/*key=total*/;



/*ATE Weighted logistic regression model*/;
proc logistic descending data=strat;
class  GE55 (descending param=ref) Male(descending param=ref ) VarPM(descending param=ref) VarLV(descending param=ref) 
VarINT(descending param=ref) VarHS(descending param=ref ) VarCM (descending param=ref) VarCD (descending param=ref) VarSVH (descending param=ref)
VarTS(descending param=ref)  LSI(descending param=ref);
Model died = LSI DSev1 DSev2 DSev1F  /risklimits; 
weight stratumwgt ;/*Specify same weight as in output dataset above*/;
run;






/*PART III Sample programs*/;

/*Part 3: Greedy Matching - With Existing Propensity Scores in the Input Data Set*/;


proc psmatch data=TWO region=CS;
class  GE55  Male VarPM VarLV VarINT VarHS VarCM  VarCD  VarSVH 
VarTS  Interv agegrp;
psdata treatvar= Interv(Treated='Yes') ps=pscore;
match method=greedy(k=1)  exact=Male exact=agegrp distance=lps caliper= 0.25;
assess  ps lps var= (VarPM VarINT VarHS VarCM  VarCD  VarSVH 
VarTS AgeP DSev1 DSev2) / plots=(barchart ) weight=none;
output out(obs=match)=matchGRDPS lps=_Lps matchid=_MatchID;

proc sort  data=matchGRDPS;
by _matchid;
run;

proc print data=matchGRDPS;
where _matchid <=5;
run;
